@@ -39,11 +39,14 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
 
 #include "update-notifier.h"
 #include "livepatch-tray.h"
 #include "update.h"
+#include "unattended-upgrade.h"
 #include "hooks.h"
+#include "reboot.h"
 #include "uevent.h"
 #include "crash.h"
 #include "release.h"
@@ -147,40 +150,6 @@ static void start_next_script(GPid pid,
 
 }
 
-void invoke(const gchar *cmd, const gchar *desktop, gboolean with_pkexec)
-{
-   GdkAppLaunchContext *context;
-   GAppInfo *appinfo;
-   GError *error = NULL;
-   static GtkWidget *w = NULL;
-
-   // pkexec
-   if(with_pkexec || FORCE_PKEXEC) {
-      invoke_with_pkexec(cmd);
-      return;
-   }
-
-   // fake window to get the current server time *urgh*
-   if (!w) {
-      w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_widget_realize (w);
-   }
-
-   // normal launch
-   context = gdk_display_get_app_launch_context (gdk_display_get_default ());
-   guint32 timestamp =  gdk_x11_get_server_time (gtk_widget_get_window (w));
-   appinfo = g_app_info_create_from_commandline(cmd, 
-						cmd, 
-						G_APP_INFO_CREATE_NONE,
-						&error);
-   gdk_app_launch_context_set_timestamp (context, timestamp);
-   if (!g_app_info_launch (appinfo, NULL, (GAppLaunchContext*)context, &error))
-      g_warning ("Launching failed: %s", error->message);
-   g_object_unref (context);
-   g_object_unref (appinfo);
-
-}
-
 void
 invoke_with_pkexec(const gchar *cmd)
 {
@@ -251,11 +220,19 @@ monitor_cb(GFileMonitor *handle,
       g_debug_inotify("apt_get_running=TRUE");
       un->apt_get_running=TRUE;
    }
+   if (g_str_has_prefix(info_uri, DPKG_INFO_DIR)) {
+      g_debug_inotify("/var/lib/dpkg/info/ updated");
+      dpkg_info_updated();
+   }
    if(strstr(info_uri, "/var/lib/update-notifier/dpkg-run-stamp") ||
       strstr(info_uri, "/var/lib/apt/periodic/update-success-stamp")) {
       g_debug_inotify("dpkg_was_run=TRUE");
       un->dpkg_was_run = TRUE;
    }
+   if(strstr(info_uri, REBOOT_FILE)) {
+      g_debug_inotify("reboot required");
+      un->reboot_pending = TRUE;
+   }
    if(strstr(info_uri, HOOKS_DIR)) {
       g_debug_inotify("new hook!");
       un->hook_pending = TRUE;
@@ -277,6 +254,7 @@ monitor_cb(GFileMonitor *handle,
  * - apt_get_running: set when apt/dpkg activity is detected (in the
  *                    lists-dir, archive-cache, or /var/lib/dpkg/status)
  * - hook_pending: we have new upgrade hook information
+ * - reboot_pending: we need to reboot
  * - crashreport_pending: we have a new crashreport
  * - livepatch_pending: livepatch status changed
  *
@@ -316,6 +294,12 @@ file_monitor_periodic_check(gpointer data)
       if(un->update_finished_timer > 0) 
 	 g_source_remove(un->update_finished_timer);
 
+      if(un->reboot_pending) {
+	 //g_print("checking reboot now\n");
+	 reboot_check (un->reboot);
+	 un->reboot_pending = FALSE;
+      }
+
       // apt must be finished when a new stamp file was written, so we
       // reset the apt_get_running time-slice field because its not
       // important anymore (it finished running)
@@ -378,7 +362,6 @@ static gboolean
 monitor_init(UpgradeNotifier *un)
 {
    int i;
-   GFileMonitor *monitor_handle;
 
    // monitor these dirs
    static const char *monitor_dirs[] = {
@@ -386,18 +369,26 @@ monitor_init(UpgradeNotifier *un)
       "/var/cache/apt/archives/", "/var/cache/apt/archives/partial/",
       HOOKS_DIR,
       CRASHREPORT_DIR,
+      DPKG_INFO_DIR,
       NULL};
    for(i=0;monitor_dirs[i] != NULL;i++) {
       if (getenv("UPSTART_SESSION") && monitor_dirs[i] == CRASHREPORT_DIR) {
          continue;
       }
-      GError *error = NULL;
-      GFile *gf = g_file_new_for_path(monitor_dirs[i]);
-      monitor_handle = g_file_monitor_directory(gf, 0, NULL, &error);
-      if(monitor_handle)
-	 g_signal_connect(monitor_handle, "changed", (GCallback)monitor_cb, un);
-      else
-	 g_warning("can not add '%s'", monitor_dirs[i]);
+      g_autoptr (GError) error = NULL;
+      g_autoptr (GFile) gf = g_file_new_for_path (monitor_dirs[i]);
+      g_autoptr (GFileMonitor) file_monitor = NULL;
+
+      if (!(file_monitor = g_file_monitor_directory (gf, 0, NULL, &error))) {
+         g_warning ("can not monitor directory '%s': %s",
+                    monitor_dirs[i], error->message);
+         continue;
+      }
+
+      g_debug ("Monitoring directory '%s'", monitor_dirs[i]);
+      g_signal_connect (file_monitor, "changed", G_CALLBACK (monitor_cb), un);
+      un->file_monitors = g_slist_append (un->file_monitors,
+                                          g_steal_pointer (&file_monitor));
    }
 
    // and those files
@@ -406,15 +397,23 @@ monitor_init(UpgradeNotifier *un)
       "/var/lib/update-notifier/dpkg-run-stamp",
       "/var/lib/apt/periodic/update-success-stamp",
       LIVEPATCH_FILE,
+      REBOOT_FILE,
       NULL};
    for(i=0;monitor_files[i] != NULL;i++) {
-      GError *error = NULL;
-      GFile *gf = g_file_new_for_path(monitor_files[i]);
-      monitor_handle = g_file_monitor_file(gf, 0, NULL, &error);
-      if(monitor_handle)
-	 g_signal_connect(monitor_handle, "changed", (GCallback)monitor_cb, un);
-      else
-	 g_warning("can not add '%s'", monitor_dirs[i]);
+      g_autoptr (GError) error = NULL;
+      g_autoptr (GFile) gf = g_file_new_for_path (monitor_files[i]);
+      g_autoptr (GFileMonitor) file_monitor = NULL;
+
+      if (!(file_monitor = g_file_monitor_file (gf, 0, NULL, &error))) {
+         g_warning ("can not monitor file '%s': %s",
+                    monitor_dirs[i], error->message);
+         continue;
+      }
+
+      g_debug ("Monitoring file '%s'", monitor_dirs[i]);
+      g_signal_connect (file_monitor, "changed", G_CALLBACK (monitor_cb), un);
+      un->file_monitors = g_slist_append (un->file_monitors,
+                                          g_steal_pointer (&file_monitor));
    }
    g_timeout_add_seconds (TIMEOUT_FAM,
 			  (GSourceFunc)file_monitor_periodic_check, un);
@@ -423,29 +422,45 @@ monitor_init(UpgradeNotifier *un)
    return TRUE;
 }
 
+static void
+monitor_uninit (UpgradeNotifier *un)
+{
+   for (GSList *l = un->file_monitors; l; l = l->next)
+      g_signal_handlers_disconnect_by_data (l->data, un);
 
-
+   g_clear_slist (&un->file_monitors, g_object_unref);
+}
 
 static gboolean
 tray_icons_init(UpgradeNotifier *un)
 {
    //g_debug_inotify("tray_icons_init");
 
-   /* new updates tray icon */
-   un->update = g_new0 (TrayApplet, 1);
-
-   // check if the updates icon should be displayed
+   /* new updates tray icon
+   /* check if the updates icon should be displayed
+    */
    if (in_admin_group() || FORCE_START) {
-      trayapplet_create(un->update, un, "software-update-available");
-      update_tray_icon_init(un->update);
-   } else
-      un->update = NULL;
+      g_autoptr (GDesktopAppInfo) software_updater_app = NULL;
+
+      software_updater_app = g_desktop_app_info_new ("update-manager.desktop");
+
+      if (software_updater_app) {
+         un->update = g_new0 (TrayApplet, 1);
+         trayapplet_create(un->update, un, "software-update-available");
+         update_tray_icon_init(un->update, software_updater_app);
+      }
+   }
 
    /* update hook icon*/
    un->hook = g_new0 (TrayApplet, 1);
    trayapplet_create(un->hook, un, "hook-notifier");
    hook_tray_icon_init(un->hook);
 
+   /* reboot required icon */
+   un->reboot = g_new0 (TrayApplet, 1);
+   trayapplet_create(un->reboot, un, "un-reboot");
+   reboot_tray_icon_init(un->reboot);
+
    /* crashreport detected icon */
    un->crashreport = g_new0 (TrayApplet, 1);
    trayapplet_create(un->crashreport, un, "apport");
@@ -456,9 +471,24 @@ tray_icons_init(UpgradeNotifier *un)
    trayapplet_create(un->livepatch, un, "livepatch");
    livepatch_tray_icon_init(un->livepatch);
 
+   /* unattended upgrades icon */
+   un->unattended_upgrade = g_new0 (TrayApplet, 1);
+   trayapplet_create (un->unattended_upgrade, un, "unattended-upgrade");
+   unattended_upgrade_tray_icon_init (un->unattended_upgrade);
+
    return FALSE; // for the tray_destroyed_cb
 }
 
+static void
+tray_icons_dispose (UpgradeNotifier *un)
+{
+   if (un->unattended_upgrade)
+      unattended_upgrade_tray_icon_dispose (un->unattended_upgrade);
+
+   // TODO: other applets
+   // TODO: actually free the TrayApplet data
+}
+
 static gboolean
 system_user(UpgradeNotifier *un)
 {
@@ -530,6 +560,32 @@ sigint_cb (gpointer user_data __attribute__ ((__unused__)))
 	return FALSE;
 }
 
+static time_t
+dpkg_info_time_since_update_internal(gboolean reset)
+{
+    static time_t last_update = 0;
+    time_t current_time;
+
+    current_time = time(NULL);
+
+    if (reset || last_update == 0)
+        last_update = current_time;
+
+    return current_time - last_update;
+}
+
+time_t
+dpkg_info_time_since_update(void)
+{
+    return dpkg_info_time_since_update_internal(/* reset = */FALSE);
+}
+
+void
+dpkg_info_updated(void)
+{
+    (void) dpkg_info_time_since_update_internal(/* reset = */TRUE);
+}
+
 
 static GOptionEntry entries[] =
 {
@@ -564,7 +620,8 @@ main (int argc, char **argv)
 	   exit(1);
 	}
 
-	notify_init("update-notifier");
+	notify_init ("update-notifier");
+
         bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
         bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
         textdomain(GETTEXT_PACKAGE);
@@ -590,7 +647,7 @@ main (int argc, char **argv)
 	g_set_application_name (_("update-notifier"));
 	gtk_window_set_default_icon_name ("update-notifier");
 
-	//g_print("starting update-notifier\n");
+	g_debug ("starting update-notifier");
 
 	/* Create the UpgradeNotifier object */
 	un = g_new0 (UpgradeNotifier, 1);
@@ -648,5 +705,9 @@ main (int argc, char **argv)
 				NULL, NULL);
 	gtk_main ();
 
+	tray_icons_dispose (un);
+	monitor_uninit (un);
+	notify_uninit ();
+
 	return 0;
 }
