@@ -2,7 +2,6 @@
 
 import hashlib
 import http.server as SimpleHTTPServer
-import io
 import os
 import random
 import shutil
@@ -12,8 +11,10 @@ import time
 import tempfile
 import unittest
 
-import mock
+from mock import patch
 from multiprocessing import Process
+import multiprocessing
+multiprocessing.set_start_method('fork')
 
 sys.path.insert(0, "data")
 sys.path.insert(1, "../data")
@@ -174,7 +175,6 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
                                       "%s.failed" % fail_hookfile)
         with open(fail_stampfile, "w"):
             pass
-        orig_failstamp_date = os.stat(fail_stampfile).st_mtime
         time.sleep(0.01)
         # create the hook file
         self._setup_hook_file(fail_hookfile)
@@ -264,18 +264,7 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         self.canary_file = "not-there.txt"
         # create the hook file
         self._setup_hook_file(hookfile)
-        stdout = io.StringIO()
-        with mock.patch('subprocess.call', return_value=100) as m_call:
-            package_data_downloader.process_download_requests()
-        expected_calls = [mock.call([
-            "/usr/lib/apt/apt-helper", "download-file",
-            "http://localhost:{}/not-there.txt".format(self.PORT),
-            "{}/partial/not-there.txt".format(
-                package_data_downloader.STAMPDIR
-            ),
-            "SHA256:{}".format(self.canary_hashsum),
-        ])]
-        self.assertEqual(expected_calls, m_call.call_args_list)
+        package_data_downloader.process_download_requests()
         self.assertTrue(os.path.exists(
             os.path.join(package_data_downloader.STAMPDIR,
                          "%s.failed" % hookfile)))
@@ -293,20 +282,10 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         # create an empty notifier file
         with open(package_data_downloader.NOTIFIER_FILE, "w"):
             pass
-        with mock.patch('subprocess.call', return_value=100) as m_call:
-            package_data_downloader.process_download_requests()
+        package_data_downloader.process_download_requests()
         # because there was a failure it shouldn't be removed
         self.assertTrue(os.path.exists(
             package_data_downloader.NOTIFIER_FILE))
-        expected_calls = [mock.call([
-            "/usr/lib/apt/apt-helper", "download-file",
-            "http://localhost:{}/not-there.txt".format(self.PORT),
-            "{}/partial/not-there.txt".format(
-                package_data_downloader.STAMPDIR
-            ),
-            "SHA256:{}".format(self.canary_hashsum),
-        ])]
-        self.assertEqual(expected_calls, m_call.call_args_list)
 
 
 class PackageDataDownloaderTestCase(unittest.TestCase):
@@ -366,7 +345,7 @@ class PackageDataDownloaderTestCase(unittest.TestCase):
             pass
         from datetime import datetime, timedelta
         # patch datetime so we think its the future
-        with mock.patch('package_data_downloader.datetime') as mock_datetime:
+        with patch('package_data_downloader.datetime') as mock_datetime:
             thefuture = (datetime.now() + timedelta(days=3))
             mock_datetime.now.return_value = thefuture
             mock_datetime.fromtimestamp.side_effect = \
